@@ -11,15 +11,6 @@
 
 #include "defs.h"
 
-/*
- * Try to turn back the IPv6 monster at the gate
- */
-#ifdef DO_NOT_USE_IPV6
-# undef INET6
-#else
-# define INET6
-#endif
-
 /*
  * Everybody needs these ANSI headers...
  */
@@ -69,6 +60,9 @@
 # define INCLUDE_PROTOTYPES
 # include <socks.h>
 #endif
+#ifdef HAVE_SYS_UN_H
+#include <sys/un.h>
+#endif
 
 /*
  * Some systems define tputs, etc in this header
@@ -157,6 +151,12 @@
 # endif
 #endif
 
+#ifdef HAVE_ATTRIBUTE_MAY_ALIAS
+#define MAY_ALIAS __attribute__((may_alias))
+#else
+#define MAY_ALIAS
+#endif
+
 /*
  * Figure out how to make alloca work
  * I took this from the autoconf documentation
@@ -372,49 +372,23 @@ struct sockaddr_storage {
 typedef int socklen_t;
 #endif
 
-#ifndef HAVE_STRUCT_SOCKADDR_IN6
-#undef INET6
-#endif
-
-#if !defined(HAVE_GETADDRINFO) || !defined(HAVE_GETNAMEINFO) || !defined(HAVE_STRUCT_ADDRINFO) || !defined(INET6)
-# define NEED_GAILIB
-# undef INET6
-# define getaddrinfo getaddrinfo__compat
-# define freeaddrinfo freeaddrinfo__compat
-# define gai_strerror gai_strerror__compat
-   struct addrinfo__compat {
-        int     ai_flags;       /* AI_PASSIVE, AI_CANONNAME */
-        int     ai_family;      /* PF_xxx */
-        int     ai_socktype;    /* SOCK_xxx */
-        int     ai_protocol;    /* 0 or IPPROTO_xxx for IPv4 and IPv6 */
-        size_t  ai_addrlen;     /* length of ai_addr */
-        char    *ai_canonname;  /* canonical name for hostname */
-        struct sockaddr *ai_addr;       /* binary address */
-        struct addrinfo__compat *ai_next;    /* next structure in linked list */
-   };
-# define addrinfo addrinfo__compat
-# define getnameinfo getnameinfo__compat
-# include "gailib.h"
-#endif
-
-
 /*
- * Define some lazy shorthand typedefs for commonly used structures
+ * C99 requires that type-punned aliases must be accessed through a union.
+ * So, that's why we do this.  Always write to 'ss' and then read back
+ * through whatever type you need.
  */
-typedef struct sockaddr 	SA;
-typedef struct sockaddr_storage	SS;
-typedef struct sockaddr_in 	ISA;
-typedef struct in_addr		IA;
-
-#ifdef INET6
-typedef struct sockaddr_in6	ISA6;
-typedef struct sockaddr_in6	I6SA;
-typedef struct in6_addr		IA6;
-typedef struct in6_addr		I6A;
+typedef union SSu {
+	struct sockaddr 	sa;
+	struct sockaddr_storage	ss;
+	struct sockaddr_in	si;
+	struct sockaddr_in6	si6;
+#ifdef HAVE_SYS_UN_H
+	struct sockaddr_un	su;
 #endif
+} SSu;
 
 typedef struct addrinfo		AI;
-typedef struct hostent		Hostent;
+
 #ifndef __no_timeval_stuff__
 typedef struct timeval		Timeval;
 #endif
@@ -457,6 +431,7 @@ typedef struct stat		Stat;
 #define FALLTHROUGH 
 #endif
 
+
 #ifndef NO_SSL
 /* Everybody needs these OpenSSL headers */
 #include <openssl/crypto.h>
