@@ -74,10 +74,13 @@ int	server_strnicmp		(const unsigned char *, const unsigned char *, size_t, int)
 #define server_stricmp(x, y, s)	server_strnicmp(x, y, UINT_MAX, s)
 int	my_strnicmp 		(const unsigned char *, const unsigned char *, size_t);
 int	my_stricmp 		(const unsigned char *, const unsigned char *);
+int     my_strncmp 		(const unsigned char *, const unsigned char *, size_t);
 int	ascii_strnicmp 		(const unsigned char *, const unsigned char *, size_t);
 int	ascii_stricmp 		(const unsigned char *, const unsigned char *);
 int	rfc1459_strnicmp 	(const unsigned char *, const unsigned char *, size_t);
 int	rfc1459_stricmp 	(const unsigned char *, const unsigned char *);
+int     alist_stricmp 		(const unsigned char *, const unsigned char *, size_t);
+
 char *  uuid4_generate 		(void);
 char *	uuid4_generate_no_dashes	(void);
 
