@@ -200,12 +200,12 @@ static char *logfile_get_targets (Logfile *log)
 	char *nicks = NULL;
 	int	i;
 
-	if (log->type == LOG_TARGETS)
+	if (log->type == LOG_TARGETS || log->type == LOG_WINDOWS)
 	{
 		for (tmp = log->targets; tmp; tmp = tmp->next)
 			malloc_strcat_wordlist(&nicks, ",", tmp->nick);
 	}
-	else if (log->type == LOG_SERVERS || log->type == LOG_WINDOWS)
+	else if (log->type == LOG_SERVERS)
 	{
 		for (i = 0; i < MAX_TARGETS; i++)
 			if (log->refnums[i] != -1)
@@ -269,17 +269,17 @@ static Logfile *	logfile_add (Logfile *log, char **args)
 
 		if (log->type == LOG_TARGETS)
 		{
-                    if (!find_in_list((List **)&log->targets, arg, !USE_WILDCARDS))
+		    if (FIND_IN_LIST_(new_w, log->targets, arg, !USE_WILDCARDS) == NULL)
                     {
                         say("Added %s to log name list", arg);
                         new_w = (WNickList *)new_malloc(sizeof(WNickList));
                         new_w->nick = malloc_strdup(arg);
-                        add_to_list((List **)&(log->targets), (List *)new_w);
+			ADD_TO_LIST_(&log->targets, new_w);
                     }
                     else
                         say("%s already on log name list", arg);
 		}
-		else if (log->type == LOG_SERVERS || log->type == LOG_WINDOWS)
+		else if (log->type == LOG_SERVERS)
 		{
 		    int refnum;
 
@@ -308,6 +308,16 @@ static Logfile *	logfile_add (Logfile *log, char **args)
 		    if (i >= MAX_TARGETS)
 			say("Could not add %d to log name list!", refnum);
 		}
+		else if (log->type == LOG_WINDOWS)
+		{
+		    if (FIND_IN_LIST_(new_w, log->targets, arg, !USE_WILDCARDS) == NULL)
+                    {
+                        say("Added %s to log window list", arg);
+                        new_w = (WNickList *)new_malloc(sizeof(WNickList));
+                        new_w->nick = malloc_strdup(arg);
+			ADD_TO_LIST_(&log->targets, new_w);
+                    }
+		}
                 arg = ptr;
         }
 
@@ -513,8 +523,8 @@ static Logfile *	logfile_on (Logfile *log, char **args)
 	}
 
 	time(&log->activity);
-	do_log(1, log->filename, &log->log);
-	log->active = 1;
+	if (do_log(1, log->filename, &log->log))
+		log->active = 1;
 	return log;
 }
 
@@ -549,7 +559,7 @@ static Logfile *	logfile_remove (Logfile *log, char **args)
 
 		if (log->type == LOG_TARGETS)
 		{
-		    if ((new_nl = (WNickList *)remove_from_list((List **)&(log->targets), arg)))
+		    if (REMOVE_FROM_LIST_(new_nl, &log->targets, arg))
 		    {
 			say("Removed %s from log target list", new_nl->nick);
 			new_free(&new_nl->nick);
@@ -706,7 +716,7 @@ BUILT_IN_COMMAND(logcmd)
 }
 
 /****************************************************************************/
-void	add_to_logs (long winref, int servref, const char *target, int level, const char *orig_str)
+void	add_to_logs (long window, int servref, const char *target, int level, const char *orig_str)
 {
 	Logfile *log;
 	int	i;
@@ -715,14 +725,53 @@ void	add_to_logs (long winref, int servref, const char *target, int level, const
 	{
 	    if (log->type == LOG_WINDOWS)
 	    {
-		for (i = 0; i < MAX_TARGETS; i++) {
-		    if (log->refnums[i] == winref) {
-			if (!mask_isset(&log->mask, level))
-				continue;
-			time(&log->activity);
-			add_to_log(log->refnum, log->log, winref, orig_str, log->mangler, log->rewrite);
-		    }
+		WNickList *	item;
+		int		matched = 0;
+
+		/* Look to see if any targets apply */
+		for (item = log->targets; item; item = item->next)
+		{
+			/* 
+			 * A log target may be a number, which must match
+			 * either the internal refnum or user refnum of 
+			 * the output window.
+			 */
+			if (is_number(item->nick))
+			{
+				int	refnum = my_atol(item->nick);
+
+				if (refnum == get_window_refnum(window) ||
+				    refnum == get_window_user_refnum(window))
+					matched = 1;
+			}
+
+			/*
+			 * Or a log target may be a string, which must match
+			 * the name of the output window (which must have a
+			 * name, natch), or the uuid of the output window.
+			 */
+			else
+			{
+				if (get_window_name(window) &&
+				    !my_stricmp(item->nick, get_window_name(window)))
+					matched = 1;
+				if (get_window_uuid(window) &&
+				    !my_stricmp(item->nick, get_window_uuid(window)))
+					matched = 1;
+			}
 		}
+
+		/* If none of the targets matched the window, punt */
+		if (!matched)
+			continue;
+
+		/* Ensure this log is logging this output level */
+		if (!mask_isset(&log->mask, level))
+			continue;
+
+		/* OK, We're good! */
+		time(&log->activity);
+		add_to_log(log->refnum, log->log, window, orig_str, log->mangler, log->rewrite);
 	    }
 
 	    if (log->type == LOG_SERVERS)
@@ -734,7 +783,7 @@ void	add_to_logs (long winref, int servref, const char *target, int level, const
 			if (!mask_isset(&log->mask, level))
 				continue;
 			time(&log->activity);
-			add_to_log(log->refnum, log->log, winref, orig_str, log->mangler, log->rewrite);
+			add_to_log(log->refnum, log->log, window, orig_str, log->mangler, log->rewrite);
 		    }
 		}
 	    }
@@ -750,12 +799,12 @@ void	add_to_logs (long winref, int servref, const char *target, int level, const
 		if (log->targets && !target)
 			continue;
 
-		if (target && !find_in_list((List **)&log->targets, target, USE_WILDCARDS))
+		if (target && ! EXISTS_IN_LIST_(log->targets, target, USE_WILDCARDS))
 			continue;
 
 		/* OK!  We want to log it now! */
 		time(&log->activity);
-		add_to_log(log->refnum, log->log, winref, orig_str, log->mangler, log->rewrite);
+		add_to_log(log->refnum, log->log, window, orig_str, log->mangler, log->rewrite);
 	    }
 	}
 }
@@ -925,8 +974,9 @@ char *logctl	(char *input)
  * The /WINDOW NUMBER command actually swaps the refnums of two windows:
  * It's possible that 'newref' isn't in use, so that's ok.
  */
-void    logfiles_swap_winrefs (int oldref, int newref)
+void    logfiles_swap_windows (int oldref, int newref)
 {
+#if 0
 	Logfile *log;
 	int	i;
 
@@ -943,10 +993,12 @@ void    logfiles_swap_winrefs (int oldref, int newref)
 				log->refnums[i] = newref;
 		}
         }
+#endif
 }
 
-void    logfiles_merge_winrefs (int oldref, int newref)
+void    logfiles_merge_windows (int oldref, int newref)
 {
+#if 0
 	Logfile *log;
 	int	i;
 
@@ -957,9 +1009,15 @@ void    logfiles_merge_winrefs (int oldref, int newref)
 
 		for (i = 0; i < MAX_TARGETS; i++)
 		{
-			if (log->refnums[i] == oldref)
-				log->refnums[i] = newref;
+			/* If it is a number, an internal refnum, switch to new internal refnum */
+			if (log->refnums[i] == get_window_refnum(oldref))
+				log->refnum[i] = get_window_refnum(newref);
+			/* If it is a number, a user refnum, switch to new user refnum */
+			else if (log->refnums[i] == get_window_user_refnum(oldref))
+				log->refnum[i] = get_window_user_refnum(newref);
+			/* If it is a string, switch to new string */
 		}
         }
+#endif
 }
 
