@@ -170,7 +170,7 @@ void	set_ssl_root_certs_location (void *stuff)
 	    *path = 0;
 
 	    if (normalize_filename(p, path))
-		(void) 0;   /* File doesn't exist or is nonsense. just do the recovery */
+		*path = 0;   /* File doesn't exist or is nonsense. just do the recovery */
 
 	    /* 
 	     * If you set it to a directory, we overrule the directory default.
@@ -336,7 +336,7 @@ static SSL_CTX	*SSL_CTX_init (int server)
 	if (ctx != NULL)
 		return ctx;
 
-	mydata_index = SSL_get_ex_new_index(0, "mydata index", NULL, NULL, NULL);
+	mydata_index = SSL_get_ex_new_index(0, malloc_strdup("mydata index"), NULL, NULL, NULL);
 	ssl_setup_locking();
 	SSL_load_error_strings();
 	SSL_library_init();
