@@ -63,7 +63,7 @@
  * Maximum number of "%" expressions in a status line format.  If you change
  * this number, you must manually change the snprintf() in make_status 
  */
-#define STATUS_FUNCTION(x) Char * x (Window *window, short map, char key)
+#define STATUS_FUNCTION(x) Char * x (int window_, short map, char key)
 #define MAX_FUNCTIONS 40
 #define MAX_STATUS_USER 39
 
@@ -140,7 +140,7 @@ static	int	defered_status_updates = 0;
 struct status_formats {
 	short	map;
 	char 	key;
-	Char	*(*callback_function)(Window *, short, char);
+	Char	*(*callback_function)(int, short, char);
 	char	**format_var;
 	int	*format_set;
 };
@@ -361,8 +361,7 @@ static void	build_status_format (Status *s, int k)
 			*format++ = '%';
 			*format++ = 's';
 
-			s->line[k].func[cp] = 
-				status_expandos[i].callback_function;
+			s->line[k].func[cp] = status_expandos[i].callback_function;
 			s->line[k].map[cp] = map;
 			s->line[k].key[cp] = key;
 			cp++;
@@ -405,13 +404,17 @@ static void	build_status_format (Status *s, int k)
  *    1. 'w' is permitted to be NULL, but 's' is never NULL
  *    2. $status_oneoff() will pass in its own 's' 
  */
-void	compile_status (Window *w, Status *s)
+void	compile_status (int window_, Status *s)
 {
 	int 	i,
 		k;
 
-	if (w)
-		debuglog("compile_status for window %d", w->refnum);
+	if (window_ > 0)
+	{
+		if (!window_is_valid(window_))
+			return;
+		debuglog("compile_status for window %d", get_window_user_refnum(window_));
+	}
 	else
 		debuglog("compile_status for global");
 
@@ -424,14 +427,14 @@ void	compile_status (Window *w, Status *s)
 		 * If we have an overriding status_format, then we parse
 		 * that out.
 		 */
-		if (w && s->line[k].raw)
+		if (window_ > 0 && s->line[k].raw)
 			build_status_format(s, k);
 
 		/*
 		 * Otherwise, If this is for a window, just copy the essential
 		 * information over from the main status lines.
 		 */
-		else if (w)
+		else if (window_ > 0)
 		{
 			s->line[k].format = malloc_strdup(main_status.line[k].format);
 			for (i = 0; i < MAX_FUNCTIONS; i++)
@@ -475,11 +478,11 @@ void	compile_status (Window *w, Status *s)
  * Return value:
  *	-1 	Something bad happened:
  *		* Status updates are forbidden (status_updates_permitted == 0)
- *		* Window or Status were NULL (what do you want me to do?)
+ *		* window was 0 or Status was NULL (what do you want me to do?)
  * 	0	The status bar was updated, but nothing changed
  *	>= 1	The status bar was updated -- you should call redraw_status()
  */
-int	make_status (Window *window, Status *status)
+int	make_status (int window_, Status *status)
 {
 	int		status_line;
 	unsigned char	buffer	    [BIG_BUFFER_SIZE + 1];
@@ -489,27 +492,28 @@ int	make_status (Window *window, Status *status)
 	size_t		save_size;
 	Screen	*	screen;
 	int		anything_changed = 0;
+	int		user_refnum;
 
 	/* Should this be a panic? */
-	if (window == NULL || status == NULL)
+	if (window_ == 0 || status == NULL)
 	{
 		debuglog("make_status -- window is null or status is null");
 		return -1;
 	}
 
+	user_refnum = get_window_user_refnum(window_);
+
 	if (!status_updates_permitted)
 	{
-		debuglog("make_status(%d) -- no status updates right now",
-				window->refnum);
+		debuglog("make_status(%d) -- no status updates right now", user_refnum);
 		defered_status_updates++;
 		return -1;
 	}
 
 	/* For hidden windows, we just pretend they're on the main screen */
-	if (!(screen = window->screen))
+	if (!(screen = get_window_screen(window_)))
 	{
-		debuglog("make_status(%d) -- updating hidden window", 
-				window->refnum);
+		debuglog("make_status(%d) -- updating hidden window", user_refnum);
 		screen = main_screen;
 	}
 
@@ -544,10 +548,10 @@ int	make_status (Window *window, Status *status)
 		else
 			panic(1, "make_status: for window [%d], status->number is [%d] and status_line "
 				"is [%d] and that makes no sense!", 
-				window->refnum, status->number, status_line);
+				user_refnum, status->number, status_line);
 
 		debuglog("make_status(%d): status line %d, number %d, line %d",
-			window->refnum, status_line, status->number, line);
+			user_refnum, status_line, status->number, line);
 
 		/*
 		 * Sanity check:  If the status format doesnt exist, dont do
@@ -555,8 +559,7 @@ int	make_status (Window *window, Status *status)
 		 */
 		if (!status->line[line].format)
 		{
-			debuglog("make_status(%d/%d): no status format",
-				window->refnum, line);
+			debuglog("make_status(%d/%d): no status format", user_refnum, line);
 			continue;
 		}
 
@@ -570,14 +573,12 @@ int	make_status (Window *window, Status *status)
 		{
 			if (status->line[line].func[i] == NULL)
 			{
-				debuglog("make_status(%d/%d/%d): Not set up",
-					window->refnum, line, i);
+				debuglog("make_status(%d/%d/%d): Not set up", line, i);
 				return -1;	/* Not set up yet */
-/* 				panic(1, "status callback null.  Window [%d], line [%d], function [%d]", window->refnum, line, i); */
+/* 				panic(1, "status callback null.  window [%d], line [%d], function [%d]", user_refnum, line, i); */
 			}
 			func_value[i] = status->line[line].func[i]
-				(window, status->line[line].map[i],
-				 status->line[line].key[i]);
+				(window_, status->line[line].map[i], status->line[line].key[i]);
 		}
 
 		/*
@@ -608,19 +609,20 @@ int	make_status (Window *window, Status *status)
 		 */
 		if (get_int_var(STATUS_DOES_EXPANDOS_VAR))
 		{
-			int  old_fs = from_server;
-			Window *old = current_window;
-			int	owd = window_display;
+			int  	old_fs = from_server;
+			int	ocw = get_window_refnum(0);
+			int	owd;
+
+			debuglog("make_status(%d/%d): expanding expandos", user_refnum, line);
+			make_window_current_informally(window_);
+			from_server = get_window_server(0);
+			owd = swap_window_display(0);
 
-			debuglog("make_status(%d/%d): expanding expandos",
-					window->refnum, line);
-			current_window = window;
-			from_server = current_window->server;
-			window_display = 0;
 			str = expand_alias(buffer, empty_string);
-			window_display = owd;
+
+			swap_window_display(owd);
 			from_server = old_fs;
-			current_window = old;
+			make_window_current_informally(ocw);
 			strlcpy(buffer, str, sizeof buffer);
 			new_free(&str);
 		}
@@ -760,8 +762,7 @@ int	make_status (Window *window, Status *status)
 		    /*
 		     * Roll the new back onto the old
 		     */
-		    debuglog("Make_status(%d/%d/%d): changed",
-				window->refnum, status_line, line);
+		    debuglog("Make_status(%d/%d/%d): changed", user_refnum, status_line, line);
 		    malloc_strcpy(&status->line[line].result, buffer);
 		    anything_changed++;
 
@@ -775,8 +776,7 @@ int	make_status (Window *window, Status *status)
 		     * because people want this thrown for invisible
 		     * windows (although they might change their minds)
 		     */
-		    do_hook(STATUS_UPDATE_LIST, "%d %d %s", 
-					window->refnum, line, buffer);
+		    do_hook(STATUS_UPDATE_LIST, "%d %d %s", user_refnum, status_line, buffer);
 		}
 	}
 
@@ -799,28 +799,31 @@ int	make_status (Window *window, Status *status)
  * Return value:
  *	-1 	Something bad happened:
  *		* Status updates are forbidden (status_updates_permitted == 0)
- *		* Window or Status were NULL (what do you want me to do?)
+ *		* window was 0 or Status was NULL (what do you want me to do?)
  *	0	The status bars were updated
  */
-int	redraw_status (Window *window, Status *status)
+int	redraw_status (int window_, Status *status)
 {
 	int	status_line, line;
 	char *	status_str;
+	int	user_refnum;
 
 	/* Should this be a panic? */
-	if (window == NULL || status == NULL)
+	if (window_ < 1 || status == NULL)
 	{
 		debuglog("redraw_status: window or status is null!");
 		return -1;
 	}
 
+	user_refnum = get_window_user_refnum(window_);
+
 	if (!status_updates_permitted)
 	{
 		debuglog("redraw_status: no status updates right now, try later");
 		defered_status_updates++;
 		return -1;
 	}
-	debuglog("redraw_status(%d): redrawing", window->refnum);
+	debuglog("redraw_status(%d): redrawing", user_refnum);
 
 	for (status_line = 0; status_line < status->number; status_line++)
 	{
@@ -839,25 +842,25 @@ int	redraw_status (Window *window, Status *status)
 		if (!(status_str = status->line[line].result))
 		{
 			debuglog("redraw_status(%d/%d/%d): no status bar",
-				window->refnum, status_line, line);
+				user_refnum, status_line, line);
 			continue;
 		}
 
-		if (dumb_mode || !foreground || !window->screen)
+		if (dumb_mode || !foreground || !get_window_screen(window_))
 		{
 			debuglog("redraw_status(%d/%d/%d): dumb/bg/hidden",
-				window->refnum, status_line, line);
+				user_refnum, status_line, line);
 			continue;
 		}
 
 		/*
 		 * Output the status line to the screen
 		 */
-		output_screen = window->screen;
-		term_move_cursor(0, window->bottom + status_line);
+		output_screen = get_window_screen(window_);
+		term_move_cursor(0, get_window_bottom(window_) + status_line);
 		output_with_count(status_str, 1, 1);
 		debuglog("redraw_status(%d/%d/%d): status redrawn",
-			window->refnum, status_line, line);
+			user_refnum, status_line, line);
 	}
 	return 0;
 }
@@ -936,15 +939,15 @@ static void	destroy_status (Status **s)
  */
 void	build_status	(void *stuff)
 {
-	Window 	*w = NULL;
+	int	window = 0;
 
 	if (!main_status_init)
 		init_status();
 
 	/* Recompile every status */
-	compile_status(NULL, &main_status);
-	while (traverse_all_windows(&w))
-		compile_status(w, &w->status);
+	compile_status(0, &main_status);
+	while (traverse_all_windows2(&window))
+		compile_status(window, get_window_status(window));
 
 	/* This forces make_status() and redraw_status() */
 	update_all_status();
@@ -982,13 +985,13 @@ int     permit_status_update (int flag)
  * This is used to get the current window on a window's screen
  */
 #if 0
-#define CURRENT_WINDOW window->screen->current_window
+#define CURRENT_WINDOW window->screen->input_window
 #endif
 
 /*
  * This tests to see if the window IS the current window on its screen
  */
-#define IS_CURRENT_WINDOW (window->screen && window->screen->current_window == window)
+#define IS_CURRENT_WINDOW (get_window_screen(window_) && get_window_screen(window_)->input_window == window_)
 
 /*
  * This tests to see if all expandoes are to appear in all status bars
@@ -1048,9 +1051,9 @@ int     permit_status_update (int flag)
  *
  */
 #define STATUS_VARS \
-	static char *my_bufferx = NULL; \
-	static int my_bufferxsize = 64; \
-	int	actual_size;
+	static char *	my_bufferx = NULL; \
+	static int 	my_bufferxsize = 64; \
+	       int	actual_size;
 
 /* After you change 'my_bufferxspace', call this to update 'my_bufferx' */
 #define CHECK \
@@ -1103,7 +1106,7 @@ STATUS_FUNCTION(status_nickname)
 {
 	STATUS_VARS
 
-	PRESS(nick_format, get_server_nickname(window->server))
+	PRESS(nick_format, get_server_nickname(get_window_server(window_)))
 	RETURN
 }
 
@@ -1125,7 +1128,7 @@ const	char	*n = NULL;
 	/*
 	 * If this window isnt connected to a server, say so.
 	 */
-	if (window->server == NOSERV)
+	if (get_window_server(window_) == NOSERV)
 		return "No Server";
 
 	/* Map 0 uses the shortname, shown when multi-connected */
@@ -1134,14 +1137,14 @@ const	char	*n = NULL;
 	/* Map 3 uses the groupname, shown at all times */
 	/* Map 4 uses the full itsname, shown at all times */
 	if (map == 0 || map == 1) 
-		n = get_server_altname(window->server, 0);
+		n = get_server_altname(get_window_server(window_), 0);
 	else if (map == 3)
-		n = get_server_group(window->server);
+		n = get_server_group(get_window_server(window_));
 	else if (map == 4)
-		n = get_server_itsname(window->server);
+		n = get_server_itsname(get_window_server(window_));
 
 	if (!n)
-		n = get_server_name(window->server);
+		n = get_server_name(get_window_server(window_));
 	if (!n)
 		return "Unknown";
 
@@ -1153,7 +1156,7 @@ STATUS_FUNCTION(status_server_status)
 {
 	const char *	state = NULL;
 
-	state = get_server_state_str(window->server);
+	state = get_server_state_str(get_window_server(window_));
 	return state ? state : "Unknown";
 }
 
@@ -1164,8 +1167,8 @@ STATUS_FUNCTION(status_network)
 {
 	const char *text = NULL;
 
-	if (window->server != NOSERV)
-		text = get_server_005(window->server, "NETWORK");
+	if (get_window_server(window_) != NOSERV)
+		text = get_server_005(get_window_server(window_), "NETWORK");
 
 	return text ? text : "Unknown";
 }
@@ -1179,7 +1182,7 @@ STATUS_FUNCTION(status_query_nick)
 	STATUS_VARS
 	const char *q;
 
-	if (!(q = get_equery_by_refnum(window->refnum)))
+	if (!(q = get_window_equery(window_)))
 		return empty_string;
 
 	PRESS(query_format, q)
@@ -1209,6 +1212,7 @@ STATUS_FUNCTION(status_notify_windows)
 	STATUS_VARS
 	int	doneone = 0;
 	char	buf2[BIG_BUFFER_SIZE];
+	int	w = 0;
 
 	/*
 	 * This only goes to a current-type window.
@@ -1221,22 +1225,19 @@ STATUS_FUNCTION(status_notify_windows)
 	 * they have been hidden and collect their refnums.
 	 */
 	*buf2 = 0;
-	window = NULL;
-	while (traverse_all_windows(&window))
+	while (traverse_all_windows2(&w))
 	{
-		if (window->notified)
+		if (get_window_notified(w))
 		{
-			char *s = NULL;
+			const char *s = NULL;
 
-			if (window->notify_name)
-				s = window->notify_name;
-			else if (window->name)
-				s = window->name;
+			if (!(s = get_window_notify_name(w)))
+				s = get_window_name(w);
 
 			if (doneone++)
 				strlcat(buf2, ",", sizeof buf2);
 			strlcat(buf2, (map == 1 && s) ? s :
-					ltoa(window->refnum), sizeof buf2);
+					ltoa(get_window_user_refnum(w)), sizeof buf2);
 		}
 	}
 
@@ -1274,12 +1275,12 @@ STATUS_FUNCTION(status_mode)
 	const char *	chan = NULL;
 
 	/* If the user has no mode format, or we're not connected, punt. */
-        if (window->server == NOSERV)
+        if (get_window_server(window_) == NOSERV)
 		return empty_string;
 
 	/* If there is a current channel, get it's mode */
-	if ((chan = get_echannel_by_refnum(window->refnum)))
-		mode = get_channel_mode(chan, window->server);
+	if ((chan = get_window_echannel(window_)))
+		mode = get_channel_mode(chan, get_window_server(window_));
 	if (!mode)
 		mode = empty_string;
 
@@ -1333,10 +1334,10 @@ STATUS_FUNCTION(status_umode)
 	/*
 	 * Punt if the window isnt connected to a server.
 	 */
-	if (window->server < 0)
+	if (get_window_server(window_) < 0)
 		return empty_string;
 
-	strlcpy(localbuf, get_umode(window->server), sizeof localbuf);
+	strlcpy(localbuf, get_umode(get_window_server(window_)), sizeof localbuf);
 	if (!*localbuf)
 		return empty_string;
 
@@ -1353,15 +1354,15 @@ STATUS_FUNCTION(status_chanop)
 	const char	*text;
 	const char *chan;
 
-	if (window->server == NOSERV ||
-           (!(chan = get_echannel_by_refnum(window->refnum))))
+	if (get_window_server(window_) == NOSERV ||
+           (!(chan = get_window_echannel(window_))))
 		return empty_string;
 	
-	if (get_channel_oper(chan, window->server) &&
+	if (get_channel_oper(chan, get_window_server(window_)) &&
 		(text = get_string_var(STATUS_CHANOP_VAR)))
 			return text;
 
-	if (get_channel_halfop(chan, window->server) &&
+	if (get_channel_halfop(chan, get_window_server(window_)) &&
 		(text = get_string_var(STATUS_HALFOP_VAR)))
 			return text;
 
@@ -1375,7 +1376,8 @@ STATUS_FUNCTION(status_ssl)
 {
 	const char *text;
 
-	if (window->server != NOSERV && get_server_ssl_enabled(window->server) &&
+	if (get_window_server(window_) != NOSERV && 
+		get_server_ssl_enabled(get_window_server(window_)) &&
 		(text = get_string_var(STATUS_SSL_ON_VAR)))
 			return text;
 	else if ((text = get_string_var(STATUS_SSL_OFF_VAR)))
@@ -1392,16 +1394,16 @@ STATUS_FUNCTION(status_hold_lines)
 {
 	STATUS_VARS
 	int	num;
-	int	interval = window->hold_interval;
+	int	interval = get_window_hold_interval(window_);
 	int	lines_held;
 
 	if (interval == 0)
 		interval = 1;		/* XXX WHAT-ever */
 
-	if (window->holding_distance_from_display_ip > window->scrollback_distance_from_display_ip)
-		lines_held = window->holding_distance_from_display_ip - window->display_lines;
+	if (get_window_holding_distance_from_display_ip(window_) > get_window_scrollback_distance_from_display_ip(window_))
+		lines_held = get_window_holding_distance_from_display_ip(window_) - get_window_display_lines(window_);
 	else
-		lines_held = window->scrollback_distance_from_display_ip - window->display_lines;
+		lines_held = get_window_scrollback_distance_from_display_ip(window_) - get_window_display_lines(window_);
 
 	if (lines_held <= 0)
 		return empty_string;
@@ -1423,14 +1425,14 @@ STATUS_FUNCTION(status_channel)
 	char 	channel[IRCD_BUFFER_SIZE + 1];
 	int	num;
 
-	if (window->server == NOSERV || !channel_format)
+	if (get_window_server(window_) == NOSERV || !channel_format)
 		return empty_string;
 
-	if (!(chan = get_echannel_by_refnum(window->refnum)))
+	if (!(chan = get_window_echannel(window_)))
 		return empty_string;
 
 	if (get_int_var(HIDE_PRIVATE_CHANNELS_VAR) && 
-	    is_channel_private(chan, window->server))
+	    is_channel_private(chan, get_window_server(window_)))
 		strlcpy(channel, "*private*", sizeof channel);
 	else
 		strlcpy(channel, chan, sizeof channel);
@@ -1453,12 +1455,12 @@ STATUS_FUNCTION(status_voice)
 	const char *text;
 	const char *chan;
 
-	if (window->server == NOSERV ||
-           (chan = get_echannel_by_refnum(window->refnum)) == NULL)
+	if (get_window_server(window_) == NOSERV ||
+           (chan = get_window_echannel(window_)) == NULL)
 		return empty_string;
 
-	if (get_channel_voice(chan, window->server) &&
-	    !get_channel_oper(chan, window->server) &&
+	if (get_channel_voice(chan, get_window_server(window_)) &&
+	    !get_channel_oper(chan, get_window_server(window_)) &&
 	    (text = get_string_var(STATUS_VOICE_VAR)))
 		return text;
 
@@ -1531,8 +1533,8 @@ STATUS_FUNCTION(status_away)
 	if (connected_to_server == 1 && !DISPLAY_ON_WINDOW)
 		return empty_string;
 
-	if (window->server != NOSERV && 
-	    get_server_away(window->server) && 
+	if (get_window_server(window_) != NOSERV && 
+	    get_server_away(get_window_server(window_)) && 
 	    (text = get_string_var(STATUS_AWAY_VAR)))
 		return text;
 
@@ -1599,8 +1601,8 @@ STATUS_FUNCTION(status_hold)
 {
 	const char *text;
 
-	if (window->holding_distance_from_display_ip > window->display_lines  ||
-	    window->scrollback_distance_from_display_ip > window->display_lines)
+	if (get_window_holding_distance_from_display_ip(window_) > get_window_display_lines(window_) ||
+	    get_window_scrollback_distance_from_display_ip(window_) > get_window_display_lines(window_))
 		if ((text = get_string_var(STATUS_HOLD_VAR)))
 			return text;
 	return empty_string;
@@ -1611,13 +1613,11 @@ STATUS_FUNCTION(status_holdmode)
 	const char *text;
 
 	/* If hold mode is on... */
-	if (window->holding_top_of_display)
+	if (get_window_hold_mode(window_))
 	{
 	    /* ... and we're not holding anything */
-	    if (window->holding_distance_from_display_ip <
-				window->display_lines  &&
-	         window->scrollback_distance_from_display_ip <
-				window->display_lines)
+	    if (get_window_holding_distance_from_display_ip(window_) < get_window_display_lines(window_) &&
+	        get_window_scrollback_distance_from_display_ip(window_) < get_window_display_lines(window_))
 	    {
 		if ((text = get_string_var(STATUS_HOLDMODE_VAR)))
 			return text;
@@ -1630,7 +1630,7 @@ STATUS_FUNCTION(status_oper)
 {
 	const char *text;
 
-	if (window->server != NOSERV && get_server_operator(window->server) &&
+	if (get_window_server(window_) != NOSERV && get_server_operator(get_window_server(window_)) &&
 	    (text = get_string_var(STATUS_OPER_VAR)) &&
 	    (connected_to_server != 1 || DISPLAY_ON_WINDOW))
 		return text;
@@ -1648,9 +1648,9 @@ STATUS_FUNCTION(status_window)
 	switch (map)
 	{
 		case 0:
-			if (!window->screen)
+			if (!get_window_screen(window_))
 				break;
-			if (number_of_windows_on_screen(window) <= 1)
+			if (number_of_windows_on_screen(get_window_screen(window_)) <= 1)
 				break;
 			/* FALLTHROUGH */
 		case 3:
@@ -1669,10 +1669,8 @@ STATUS_FUNCTION(status_refnum)
 	STATUS_VARS
 	const char *value;
 
-	if (window->name)
-		value = window->name;
-	else
-		value = ltoa(window->refnum);
+	if (!(value = get_window_name(window_)))
+		value = ltoa(get_window_user_refnum(window_));
 
 	PRESS("%s", value)
 	RETURN
@@ -1682,7 +1680,7 @@ STATUS_FUNCTION(status_refnum_real)
 {
 	STATUS_VARS
 
-	PRESS("%s", ltoa(window->refnum))
+	PRESS("%d", get_window_user_refnum(window_));
 	RETURN
 }
 
@@ -1707,9 +1705,13 @@ STATUS_FUNCTION(status_null_function)
  */
 STATUS_FUNCTION(status_dcc)
 {
+	Mask	mask;
+
+	get_window_mask(window_, &mask);
+
 	if ((mask_isset(&current_window_mask, LEVEL_DCC) && 
 				IS_CURRENT_WINDOW) ||
-	    (mask_isset(&window->window_mask, LEVEL_DCC)))
+	    (mask_isset(&mask, LEVEL_DCC)))
 		return DCC_get_current_transfer();
 
 	return empty_string;
@@ -1749,10 +1751,11 @@ STATUS_FUNCTION(status_position)
 	static char my_buffer[81];
 
 	snprintf(my_buffer, sizeof my_buffer, "(%d/%d/%d-%d-%d)", 
-			window->scrolling_distance_from_display_ip,
-			window->holding_distance_from_display_ip,
-			window->scrollback_distance_from_display_ip,
-			window->display_lines, window->cursor);
+			get_window_scrolling_distance_from_display_ip(window_),
+			get_window_holding_distance_from_display_ip(window_),
+			get_window_scrollback_distance_from_display_ip(window_),
+			get_window_display_lines(window_),
+			get_window_cursor(window_));
 	return my_buffer;
 }
 
@@ -1764,7 +1767,7 @@ STATUS_FUNCTION(status_scrollback)
 {
 	const char *stuff;
 
-	if (window->scrollback_top_of_display &&
+	if (get_window_scrollback_top_of_display_exists(window_) &&
 	    (stuff = get_string_var(STATUS_SCROLLBACK_VAR)))
 		return stuff;
 	else
@@ -1775,20 +1778,20 @@ STATUS_FUNCTION(status_scroll_info)
 {
 	static char my_buffer[81];
 
-	if (!window->scrollback_top_of_display)
+	if (!get_window_scrollback_top_of_display_exists(window_))
 		return empty_string;
 
 	snprintf(my_buffer, sizeof my_buffer, " (Scroll: %d of %d)", 
-			window->scrollback_distance_from_display_ip,
-			window->display_buffer_size - 1);
+			get_window_scrollback_distance_from_display_ip(window_),
+			get_window_display_buffer_size(window_) - 1);
 	return my_buffer;
 }
 
 
 STATUS_FUNCTION(status_windowspec)
 {
-	if (window->status.special)
-		return window->status.special;
+	if (get_window_status(window_)->special)
+		return get_window_status(window_)->special;
 	else
 		return empty_string;
 }
@@ -1797,8 +1800,8 @@ STATUS_FUNCTION(status_window_prefix)
 {
 	if (IS_CURRENT_WINDOW)
 	{
-		if (window->status.prefix_when_current)
-			return window->status.prefix_when_current;
+		if (get_window_status(window_)->prefix_when_current)
+			return get_window_status(window_)->prefix_when_current;
 		else if (get_string_var(STATUS_PREFIX_WHEN_CURRENT_VAR))
 			return get_string_var(STATUS_PREFIX_WHEN_CURRENT_VAR);
 		else
@@ -1806,8 +1809,8 @@ STATUS_FUNCTION(status_window_prefix)
 	}
 	else
 	{
-		if (window->status.prefix_when_not_current)
-			return window->status.prefix_when_not_current;
+		if (get_window_status(window_)->prefix_when_not_current)
+			return get_window_status(window_)->prefix_when_not_current;
 		else if (get_string_var(STATUS_PREFIX_WHEN_NOT_CURRENT_VAR))
 			return get_string_var(STATUS_PREFIX_WHEN_NOT_CURRENT_VAR);
 		else
@@ -1835,7 +1838,7 @@ STATUS_FUNCTION(status_swappable)
 {
 	const char *stuff;
 
-	if (!window->swappable && 
+	if (!get_window_swappable(window_) &&
 	    (stuff = get_string_var(STATUS_NOSWAP_VAR)))
 		return stuff;
 	else
@@ -1848,20 +1851,11 @@ STATUS_FUNCTION(status_activity)
 	char *result;
 	static char retval[80];
 
-	if (window->current_activity == 0)
+	if (!get_window_current_activity(window_))
 		return empty_string;
 
-	format = window->activity_format[window->current_activity];
-	if (!format || !*format)
-		format = window->activity_format[0];
-	if (!format || !*format)
-		return empty_string;
-
-	data = window->activity_data[window->current_activity];
-	if (!data || !*data)
-		data = window->activity_data[0];
-	if (!data || !*data)
-		data = empty_string;
+	format = get_window_current_activity_format(window_);
+	data = get_window_current_activity_data(window_);
 
 	result = expand_alias(format, data);
 	strlcpy(retval, result, sizeof(retval));
@@ -1873,19 +1867,19 @@ BUILT_IN_FUNCTION(function_status_oneoff, input)
 {
 	Status *s;
 	char *	windesc;
-	Window	*w;
 	char	*retval;
+	int	window;
 
 	GET_FUNC_ARG(windesc, input);
 
-	if (!(w = get_window_by_desc(windesc)))
+	if ((window = lookup_window(windesc)) < 1)
 		RETURN_EMPTY;
 
 	s = new_status();
 	malloc_strcpy(&s->line[0].raw, input);
 
-	compile_status(w, s);
-	make_status(w, s);
+	compile_status(window, s);
+	make_status(window, s);
 
 	retval = denormalize_string(s->line[0].result);
 	destroy_status(&s);
